"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from anncsu.common.errors import AnncsuBaseError as AnncsuError
from anncsu.common.sdk.types import BaseModel
import httpx
from typing import Optional


class EsisteOdonimoGetPathParamInternalServerErrorData(BaseModel):
    title: Optional[str] = None

    detail: Optional[str] = None


class EsisteOdonimoGetPathParamInternalServerError(AnncsuError):
    r"""Internal Server Error"""

    data: EsisteOdonimoGetPathParamInternalServerErrorData

    def __init__(
        self,
        data: EsisteOdonimoGetPathParamInternalServerErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        message = body or raw_response.text
        super().__init__(message, raw_response, body)
        self.data = data


class EsisteOdonimoGetPathParamUnprocessableEntityErrorData(BaseModel):
    title: Optional[str] = None

    detail: Optional[str] = None


class EsisteOdonimoGetPathParamUnprocessableEntityError(AnncsuError):
    r"""Unprocessable Entity - error in json"""

    data: EsisteOdonimoGetPathParamUnprocessableEntityErrorData

    def __init__(
        self,
        data: EsisteOdonimoGetPathParamUnprocessableEntityErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        message = body or raw_response.text
        super().__init__(message, raw_response, body)
        self.data = data


class EsisteOdonimoGetPathParamMethodNotAllowedErrorData(BaseModel):
    title: Optional[str] = None

    detail: Optional[str] = None


class EsisteOdonimoGetPathParamMethodNotAllowedError(AnncsuError):
    r"""Method Not Allowed"""

    data: EsisteOdonimoGetPathParamMethodNotAllowedErrorData

    def __init__(
        self,
        data: EsisteOdonimoGetPathParamMethodNotAllowedErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        message = body or raw_response.text
        super().__init__(message, raw_response, body)
        self.data = data


class EsisteOdonimoGetPathParamBadRequestErrorData(BaseModel):
    title: Optional[str] = None

    detail: Optional[str] = None


class EsisteOdonimoGetPathParamBadRequestError(AnncsuError):
    r"""Bad Request"""

    data: EsisteOdonimoGetPathParamBadRequestErrorData

    def __init__(
        self,
        data: EsisteOdonimoGetPathParamBadRequestErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        message = body or raw_response.text
        super().__init__(message, raw_response, body)
        self.data = data
